from flask import render_template, redirect, url_for, flash, Blueprint
from sqlalchemy.exc import OperationalError
from flask_admin.contrib.sqla import ModelView
from celery import states

from database import app, admin, db
from flask_covid19.blueprints.app_mq.application_workers import celery
from flask_covid19 import who_service, owid_service
from flask_covid19 import rki_service, rki_vaccination_service
from flask_covid19 import ecdc_service, divi_service
from flask_covid19 import app_admin_service, app_user_service
from flask_covid19 import RkiData, RkiMeldedatum
from flask_covid19 import RkiImport
from flask_covid19 import ApplicationPage

drop_and_create_data_again = True

app_rki = Blueprint(
    'rki', __name__, template_folder='templates', url_prefix='/rki/cases')


admin.add_view(ModelView(RkiImport, db.session, category="RKI Cases"))
admin.add_view(ModelView(RkiData, db.session, category="RKI Cases"))


# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------


@app_rki.route('/info')
def url_rki_info():
    page_info = ApplicationPage('RKI', "Info")
    return render_template(
        'rki/rki_info.html',
        page_info=page_info)


@app_rki.route('/tasks')
def url_rki_tasks():
    page_info = ApplicationPage('RKI', "Tasks")
    return render_template(
        'rki/rki_tasks.html',
        page_info=page_info)


@app_rki.route('/imported/page/<int:page>')
@app_rki.route('/imported')
def url_rki_imported(page=1):
    page_info = ApplicationPage('RKI', "Last Import")
    try:
        page_data = RkiImport.get_all_as_page(page)
    except OperationalError:
        flash("No data in the database.")
        page_data = None
    return render_template(
        'rki/rki_imported.html',
        page_data=page_data,
        page_info=page_info)


@app_rki.route('/test/page/<int:page>')
@app_rki.route('/test')
def url_rki_test(page=1):
    page_info = ApplicationPage('RKI', "TEST")
    try:
        page_data = RkiImport.get_all_as_page(page)
    except OperationalError:
        flash("No data in the database.")
        page_data = None
    return render_template(
        'rki/rki_test.html',
        page_data=page_data,
        page_info=page_info)


@app_rki.route('/date_reported/all/page/<int:page>')
@app_rki.route('/date_reported/all')
def url_rki_date_reported_all(page: int = 1):
    page_info = ApplicationPage('RKI', "Date Reported", "All")
    try:
        page_data = RkiMeldedatum.get_all_as_page(page)
    except OperationalError:
        flash("No date_reported in the database.")
        page_data = None
    return render_template(
        'rki/date_reported/rki_date_reported_all.html',
        page_data=page_data,
        page_info=page_info)


# ------------------------------------------------------------------------
#  Celery TASKS
# ------------------------------------------------------------------------

@celery.task(bind=True)
def task_rki_import(self):
    self.update_state(state=states.STARTED)
    rki_service.import_file()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_import_only)"
    return result


@celery.task(bind=True)
def task_rki_full_update_dimensiontables(self):
    self.update_state(state=states.STARTED)
    rki_service.full_update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_full_update_dimensiontables)"
    return result


@celery.task(bind=True)
def task_rki_update_dimensiontables(self):
    self.update_state(state=states.STARTED)
    rki_service.update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_update_dimensiontables)"
    return result


@celery.task(bind=True)
def task_rki_full_update_facttable(self):
    self.update_state(state=states.STARTED)
    rki_service.full_update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_full_update_facttable)"
    return result


@celery.task(bind=True)
def task_rki_update_facttable(self):
    self.update_state(state=states.STARTED)
    rki_service.update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_update_facttable)"
    return result


@celery.task(bind=True)
def task_rki_full_update_starschema(self):
    self.update_state(state=states.STARTED)
    rki_service.update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_full_update_starschema)"
    return result


@celery.task(bind=True)
def task_rki_update_starschema(self):
    self.update_state(state=states.STARTED)
    rki_service.update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_update_starschema)"
    return result

# ------------------------------------------------------------------------
#  URL Routes for Celery TASKS
# ------------------------------------------------------------------------


@app_rki.route('/task/download')
def url_task_rki_download():
    app.logger.info("url_task_rki_download [start]")
    flash("url_task_rki_download [start]")
    rki_service.download()
    flash("url_task_rki_download [done]")
    app.logger.info("url_task_rki_download [done]")
    return redirect(url_for('rki.url_rki_tasks'))


@app_rki.route('/task/import')
def url_task_rki_import():
    app.logger.info("url_task_rki_import [start]")
    task_rki_import.apply_async()
    flash(message="url_task_rki_download [async start]", category='warning')
    app.logger.warn("task_rki_import [async start]")
    return redirect(url_for('rki.url_rki_tasks'))


@app_rki.route('/task/update/full/dimension-tables')
def url_task_rki_full_update_dimensiontables():
    app.logger.info("url_task_rki_full_update_dimensiontables [start]")
    task_rki_full_update_dimensiontables.apply_async()
    flash(message="task_rki_full_update_dimensiontables [async start]", category='warning')
    app.logger.warn("task_rki_full_update_dimensiontables [async start]")
    return redirect(url_for('rki.url_rki_tasks'))


@app_rki.route('/task/update/dimension-tables')
def url_task_rki_update_dimensiontables():
    app.logger.info("url_task_rki_update_dimensiontables [start]")
    task_rki_update_dimensiontables.apply_async()
    flash(message="task_rki_update_dimensiontables [async start]", category='warning')
    app.logger.warn("task_rki_update_dimensiontables [async start]")
    return redirect(url_for('rki.url_rki_tasks'))


@app_rki.route('/task/update/full/fact-table')
def url_task_rki_full_update_facttable():
    app.logger.info("url_rki_task_update_facttable_incremental_only [start]")
    task_rki_full_update_facttable.apply_async()
    flash(message="task_rki_full_update_facttable [async start]", category='warning')
    app.logger.warn("task_rki_full_update_facttable [async start]")
    return redirect(url_for('rki.url_rki_tasks'))


@app_rki.route('/task/update/fact-table')
def url_task_rki_update_facttable():
    app.logger.info("url_task_rki_update_facttable [start]")
    task_rki_update_facttable.apply_async()
    flash(message="task_rki_update_facttable [async start]", category='warning')
    app.logger.warn("task_rki_update_facttable [async start]")
    return redirect(url_for('rki.url_rki_tasks'))


@app_rki.route('/task/update/full/star_schema')
def url_task_rki_full_update_starschema():
    app.logger.info("url_task_rki_full_update_starschema [start]")
    flash("url_task_rki_download [start]")
    rki_service.download()
    flash("url_task_rki_download [done]")
    task_rki_full_update_starschema.apply_async()
    flash(message="task_rki_full_update_starschema [async start]", category='warning')
    app.logger.warn("task_rki_full_update_starschema [async start]")
    return redirect(url_for('rki.url_rki_tasks'))


@app_rki.route('/task/update/star_schema')
def url_task_rki_update_starschema():
    app.logger.info("url_task_rki_update_starschema [start]")
    flash("url_task_rki_download [start]")
    rki_service.download()
    flash("url_task_rki_download [done]")
    task_rki_update_starschema.apply_async()
    flash(message="task_rki_update_starschema [async start]", category='warning')
    app.logger.warn("task_rki_update_starschemas [async start]")
    return redirect(url_for('rki.url_rki_tasks'))
