from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger
from flask_admin.contrib.sqla import ModelView

from database import admin, db
from flask_covid19 import who_service, owid_service
from flask_covid19 import rki_service, rki_vaccination_service
from flask_covid19 import ecdc_service, divi_service
from flask_covid19 import app_admin_service, app_user_service
from flask_covid19.blueprints.app_mq.application_workers import celery

from flask_covid19 import RkiVaccinationData, RkiVaccinationDateReported
from flask_covid19 import RkiVaccinationImport
from flask_covid19 import ApplicationPage


app_rki_vaccination = Blueprint('rki_vaccination', __name__, template_folder='templates', url_prefix='/rki/vaccination')

admin.add_view(ModelView(RkiVaccinationImport, db.session, category="RKI Vaccination"))
admin.add_view(ModelView(RkiVaccinationDateReported, db.session, category="RKI Vaccination"))
admin.add_view(ModelView(RkiVaccinationData, db.session, category="RKI Vaccination"))


# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------

@app_rki_vaccination.route('/info')
def url_vaccination_info():
    page_info = ApplicationPage('Vaccination', "Info")
    return render_template(
        'rki_vaccination/rki_vaccination_info.html',
        page_info=page_info)


@app_rki_vaccination.route('/tasks')
def url_vaccination_tasks():
    page_info = ApplicationPage('Vaccination', "Tasks")
    return render_template(
        'rki_vaccination/rki_vaccination_tasks.html',
        page_info=page_info)


@app_rki_vaccination.route('/imported/page/<int:page>')
@app_rki_vaccination.route('/imported')
def url_vaccination_imported(page=1):
    page_info = ApplicationPage('Vaccination', "Data: Germany Timeline imported")
    page_data = RkiVaccinationImport.get_all_as_page(page)
    return render_template(
        'rki_vaccination/imported/rki_vaccination_imported.html',
        page_data=page_data,
        page_info=page_info)


@app_rki_vaccination.route('/data/page/<int:page>')
@app_rki_vaccination.route('/data')
def url_vaccination_data(page=1):
    page_info = ApplicationPage('Vaccination', "Data: Germany Timeline")
    page_data = RkiVaccinationData.get_all_as_page(page)
    return render_template(
        'rki_vaccination/data/rki_vaccination_data.html',
        page_data=page_data,
        page_info=page_info)


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_vaccination_import(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_europe_import_only [OK] ")
    logger.info("------------------------------------------------------------")
    rki_vaccination_service.import_file()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_europe_import_only)"
    return result


@celery.task(bind=True)
def task_vaccination_full_update_dimension_tables(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_full_update_dimension_tables [OK] ")
    logger.info("------------------------------------------------------------")
    rki_vaccination_service.full_update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_full_update_dimension_tables)"
    return result

@celery.task(bind=True)
def task_vaccination_update_dimension_tables(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_europe_update_dimension_tables_only [OK] ")
    logger.info("------------------------------------------------------------")
    rki_vaccination_service.update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_europe_update_dimension_tables_only)"
    return result


@celery.task(bind=True)
def task_vaccination_update_facttable(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_europe_update_fact_table_incremental_only [OK] ")
    logger.info("------------------------------------------------------------")
    rki_vaccination_service.update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_europe_update_fact_table_incremental_only)"
    return result


@celery.task(bind=True)
def task_vaccination_full_update_facttable(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_europe_update_fact_table_initial_only [OK] ")
    logger.info("------------------------------------------------------------")
    rki_vaccination_service.full_update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_europe_update_fact_table_initial_only)"
    return result


@celery.task(bind=True)
def task_vaccination_update_starschema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_europe_update_star_schema_incremental [OK] ")
    logger.info("------------------------------------------------------------")
    rki_vaccination_service.full_update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_europe_update_star_schema_incremental)"
    return result


@celery.task(bind=True)
def task_vaccination_full_update_starschema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_europe_update_star_schema_initial [OK] ")
    logger.info("------------------------------------------------------------")
    rki_vaccination_service.full_update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_europe_update_star_schema_initial)"
    return result


# ----------------------------------------------------------------------------------------------------------------
# URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@app_rki_vaccination.route('/task/download')
def url_task_vaccination_download():
    flash("url_vaccination_task_download_only started")
    rki_vaccination_service.download()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_rki_vaccination.route('/task/import')
def url_task_vaccination_import():
    flash("url_vaccination_task_import_only started")
    task_vaccination_import.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_rki_vaccination.route('/task/update/full/dimension-tables')
def url_task_vaccination_full_update_dimension_tables():
    flash("url_vaccination_task_update_dimensiontables_only started")
    task_vaccination_full_update_dimension_tables.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_rki_vaccination.route('/task/update/dimension-tables')
def url_task_vaccination_update_dimension_tables():
    flash("url_vaccination_task_update_dimensiontables_only started")
    task_vaccination_update_dimension_tables.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_rki_vaccination.route('/task/update/fact-table/incremental/only')
def url_task_vaccination_update_facttable():
    flash("url_vaccination_task_update_facttable_incremental_only started")
    task_vaccination_update_facttable.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_rki_vaccination.route('/task/update/fact-table/initial/only')
def url_task_vaccination_full_update_facttable():
    flash("url_vaccination_task_update_facttable_initial_only started")
    task_vaccination_full_update_facttable.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_rki_vaccination.route('/task/update/star_schema/initial')
def url_task_vaccination_full_update_starschema():
    flash("url_vaccination_task_update_star_schemainitial started")
    rki_vaccination_service.download()
    task_vaccination_full_update_starschema.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_rki_vaccination.route('/task/update/star_schema/incremental')
def url_task_vaccination_update_starschema():
    flash("url_vaccination_task_update_starschema_incremental started")
    rki_vaccination_service.download()
    task_vaccination_update_starschema.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))
